# -*- coding: utf-8 -*-
import re
from importlib import import_module

MODULE_CACHE = {}
CLASS_CACHE = {}
FUNC_CACHE = {}


def load_module(module_name):
    try:
        return MODULE_CACHE[module_name]
    except KeyError:
        pass

    MODULE_CACHE[module_name] = import_module(module_name)
    return MODULE_CACHE[module_name]


def load_class(module_name, class_name):
    if not class_name:
        return ''

    key = module_name + '::' + class_name
    try:
        return CLASS_CACHE[key]['name']
    except KeyError:
        pass

    module = MODULE_CACHE[module_name]

    text = class_name
    if class_name and hasattr(module, class_name):
        module_class = getattr(module, class_name)
        if hasattr(module_class, '__doc__') and module_class.__doc__:
            text = module_class.__doc__

    CLASS_CACHE[key] = {
        'name': text,
        'class': module_class
    }
    return text


def load_func(module_name, class_name, func_name):
    key = module_name
    class_key = module_name + '::' + class_name
    if class_name:
        key = class_key

    key += '::' + func_name

    try:
        return FUNC_CACHE[key]['name']
    except KeyError:
        pass

    module = MODULE_CACHE[module_name]
    module_class = None
    class_text = ''
    try:
        module_class = CLASS_CACHE[class_key]['class']
    except KeyError:
        pass

    text = func_name
    module_func = None
    if module_class and hasattr(module_class, func_name):
        module_func = getattr(module_class, func_name)
    elif hasattr(module, func_name):
        module_func = getattr(module, func_name)

    if module_func and hasattr(module_func, '__doc__') and module_func.__doc__:
        text = module_func.__doc__
        text = text.strip().replace('\n', ' ')
        text = re.sub(' +', ' ', text)

    FUNC_CACHE[key] = {
        'name': text,
        'func': module_func
    }
    return text


def load_test_info(nodeid):
    data = nodeid.split('::')
    file_name = data[0]
    class_name = ''
    func_name = ''
    if len(data) > 2:
        class_name = data[1]
        func_name = data[2]
    else:
        func_name = data[1]

    module_name = file_name.replace('.py', '').replace('/', '.').replace('\\', '.')
    load_module(module_name)
    class_text = load_class(module_name, class_name)
    func_text = load_func(module_name, class_name, func_name)

    text = func_text
    if class_text:
        text = class_text + ' :: ' + text

    return file_name, text



# -*- coding: utf-8 -*-
from .reporter import logstart_replacer, report_replacer


def pytest_addoption(parser):
    group = parser.getgroup('reporting')
    group.addoption(
        '--mocha',
        action='store_true',
        dest='mocha',
        help='Enable test reporter in mochajs format'
    )
    group.addoption(
        '--mocha-force-disable',
        action='store_true',
        dest='mocha_disable',
        help='Force disable mochajs reporter'
    )


def pytest_configure(config):
    if config.option.mocha and not config.option.mocha_disable:
        import imp
        import _pytest
        _pytest.terminal.TerminalReporter \
            .pytest_runtest_logstart = logstart_replacer
        _pytest.terminal.TerminalReporter \
            .pytest_runtest_logreport = report_replacer
        imp.reload(_pytest)



# -*- coding: utf-8 -*-
from setuptools import setup

with open('README.md') as f:
    long_description = f.read()

setup(
    name='pytest-mocha',
    version='0.1.0',
    description='pytest plugin to display test execution output like a mochajs',
    long_description=long_description,
    author='Rudinei Goi Roecker',
    author_email='rudinei.roecker@gmail.com',
    url='https://github.com/rudineirk/pytest-mocha',
    license='MIT',
    packages=['pytest_mocha'],
    install_requires=[
        'colorama>=0.3.9',
    ],
    entry_points={
        'pytest11': ['pytest_mocha = pytest_mocha.plugin']
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
    ],
)



# -*- coding: utf-8 -*-
from colorama import Fore, Style

from .loader import load_test_info

STORAGE = {
    'parents': {}
}
STATUS_ICONS = {
    'passed': '✓',
    'failed': '✖',
    'skipped': '!'
}
COLORS = {
    'passed': Fore.GREEN,
    'failed': Fore.RED,
    'skipped': Fore.YELLOW
}


def logstart_replacer(self, nodeid, location):
    """Signal the start of running a single test item.
    Hook has to be disabled because additional information
    may break output formatting.
    """


def report_replacer(self, report):
    res = self.config.hook.pytest_report_teststatus(report=report)
    category, letter, word = res
    self.stats.setdefault(category, []).append(report)
    if not letter and not word:
        return

    file_name, text = load_test_info(report.nodeid)
    orig_text = text
    text = [x.strip() for x in text.split('::')]
    parents = []
    if len(text) >= 1:
        parents.extend(text[:-1])
    if not parents:
        parents = [file_name]

    text = text[-1]

    indent = '    '
    test_indent = indent
    tmp_keys = []
    upper_parent_changed = False
    for idx, parent in enumerate(parents):
        tmp_keys.append(idx)
        test_indent = indent * (idx + 1)
        old_parent = STORAGE['parents'].get(idx, None)
        if not upper_parent_changed and parent == old_parent:
            continue

        upper_parent_changed = True
        STORAGE['parents'][idx] = parent
        if idx == 0 and parent != file_name:
            parent = parent + ' :: ' + file_name

        self._tw.line()
        self._tw.write(indent * idx + parent)

    for key in list(STORAGE['parents']):
        if key not in tmp_keys:
            STORAGE['parents'].pop(key)

    self._tw.line()
    self._tw.write("{color}{indent}{icon} {text}{reset}".format(
        color=COLORS[category],
        indent=test_indent,
        icon=STATUS_ICONS[category],
        text=text,
        reset=Style.RESET_ALL
    ))
